require "behaviours/follow"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/panic"

local BrainCommon = require "brains/braincommon"
local MIN_FOLLOW_DIST = 0
local MAX_FOLLOW_DIST = 6
local TARGET_FOLLOW_DIST = 6

local MAX_WANDER_DIST = 20



local LEASH_RETURN_DIST = 10
local LEASH_MAX_DIST = 30
local GO_HOME_DIST = 20
local START_RUN_DIST = 3
local STOP_RUN_DIST = 5
local MAX_CHASE_TIME = 10
local MAX_CHASE_DIST = 30
local SEE_LIGHT_DIST = 20
local TRADE_DIST = 20
local SEE_TREE_DIST = 15
local SEE_TARGET_DIST = 20
local SEE_FOOD_DIST = 10

local SEE_BURNING_HOME_DIST_SQ = 20*20

local COMFORT_LIGHT_LEVEL = 0.3

local KEEP_CHOPPING_DIST = 10

local RUN_AWAY_DIST = 5
local STOP_RUN_AWAY_DIST = 8

local function GoHomeAction(inst)
    if inst.components.follower.leader ~= nil  then
        return 
    end
    local homePos = inst.components.knownlocations:GetLocation("home")
    return homePos ~= nil
        and BufferedAction(inst, nil, ACTIONS.WALKTO, nil, homePos)
        or nil
end

local function ShouldGoHome(inst)
	if inst.components.follower.leader ~= nil  then
        return 
    end
    local homePos = inst.components.knownlocations:GetLocation("home")
    return homePos ~= nil and inst:GetDistanceSqToPoint(homePos:Get()) > GO_HOME_DIST * GO_HOME_DIST
end

local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function FindFoodAction(inst)
    if inst.sg:HasStateTag("busy") then
        return
    end

    if inst.components.inventory ~= nil and inst.components.eater ~= nil then
        local target = inst.components.inventory:FindItem(function(item) return inst.components.eater:CanEat(item) end)
        if target ~= nil then
            return BufferedAction(inst, target, ACTIONS.EAT)
        end
    end

    local time_since_eat = inst.components.eater:TimeSinceLastEating()
    if time_since_eat ~= nil and time_since_eat <= 2 then
        return
    end

    local noveggie = time_since_eat ~= nil and time_since_eat < 4

    local target = FindEntity(inst,
        SEE_FOOD_DIST,
        function(item)
            return item:GetTimeAlive() >= 8
                and item.prefab ~= "mandrake"
                and item.components.edible ~= nil
                and (not noveggie or item.components.edible.foodtype == FOODTYPE.MEAT)
                and item:IsOnValidGround()
                and inst.components.eater:CanEat(item)
        end,
        nil,
        { "outofreach" }
    )
    if target ~= nil then
        return BufferedAction(inst, target, ACTIONS.EAT)
    end

    target = FindEntity(inst,
        SEE_FOOD_DIST,
        function(item)
            return item.components.shelf ~= nil
                and item.components.shelf.itemonshelf ~= nil
                and item.components.shelf.cantakeitem
                and item.components.shelf.itemonshelf.components.edible ~= nil
                and (not noveggie or item.components.shelf.itemonshelf.components.edible.foodtype == FOODTYPE.MEAT)
                and item:IsOnValidGround()
                and inst.components.eater:CanEat(item.components.shelf.itemonshelf)
        end,
        nil,
        { "outofreach" }
    )
    if target ~= nil then
        return BufferedAction(inst, target, ACTIONS.TAKEITEM)
    end
end


local NukoBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)
local function is_bullet(item)
    return item and string.find(item.prefab,'bullet')
end

local function Killer(inst)
	local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, 10, {"nuko_killer"})
	for k,v in pairs(ents) do 
		if v:HasTag("nuko_killer" ) then print("发现凶手！") return true end 
	end
	return false 
end

local function Monster(inst)
	local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, 10, {"monster"})
	for k,v in pairs(ents) do 
		if v:HasTag("monster" ) then  print("发现怪物！") return true end 
	end
	return false
end
function NukoBrain:OnStart()
	
    local root = 
    PriorityNode({
        WhileNode( function() return self.inst.components.hauntable and self.inst.components.hauntable.panic end, "PanicHaunted", Panic(self.inst)),
        WhileNode( function() return self.inst.components.health.takingfiredamage end, "OnFire", Panic(self.inst)),
		WhileNode(function() return true end, "MonsterNearBy",
                ChattyNode(self.inst, "nui~~~~",
                    RunAway(self.inst, "monster",7, 12))),
		WhileNode(function() return true end, "MonsterNearBy",
                ChattyNode(self.inst, "nui~~~~",
                    RunAway(self.inst, "nuko_killer",7, 12))),
		WhileNode(function() return TheWorld.state.isnight end, "sleep",
        ChattyNode(self.inst, "nui~~~~~~",
            DoAction(self.inst, function (inst)inst.components.sleeper:GoToSleep() end , "sleep", true))),
		ChattyNode(self.inst, "nui~~~~",
                DoAction(self.inst, FindFoodAction )),
        Follow(self.inst, function() 
		local bullter = nil
		local x, y, z = self.inst.Transform:GetWorldPosition()     
		local ents = TheSim:FindEntities(x, y, z, 30,{"player"}) 
			for k,v in pairs(ents) do 
				if v.components.inventory ~= nil and
					v:IsNear(self.inst, TUNING.BUNNYMAN_SEE_MEAT_DIST) and
					v.components.inventory:FindItem(is_bullet) ~= nil 
					then 
					bullter = v
				end 
			end
			
		self.inst.components.follower:SetLeader(bullter)
		return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
		
        FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
		
		
        Wander(self.inst, function()   return self.inst.components.knownlocations:GetLocation("home")   end, MAX_WANDER_DIST),
        
    }, .25)
    self.bt = BT(self.inst, root)
end

return NukoBrain