local assets =
{
    Asset("ANIM", "anim/nuko.zip"),
	Asset("SOUND", "sound/nuko.fsb"),

   -- Asset("MINIMAP_IMAGE", "chester"),
}

local prefabs =
{
    "globalmapiconunderfog",
}

local brain = require "brains/nukobrain"

local sounds =
{
    hurt = "nuko/nuko/naive_0",
    pant = "nuko/nuko/naive_0",
    death = "nuko/nuko/naive_0",
    open = "nuko/nuko/naive_0",
    close = "nuko/nuko/naive_0",
    pop = "nuko/nuko/naive_0",
    boing = "nuko/nuko/naive_0",
    lick = "nuko/nuko/naive_0",
}

local WAKE_TO_FOLLOW_DISTANCE = 14
local SLEEP_NEAR_LEADER_DISTANCE = 7

local function ShouldWakeUp(inst)
    return DefaultWakeTest(inst) and not inst.components.follower:IsNearLeader(WAKE_TO_FOLLOW_DISTANCE)
end

local function ShouldSleep(inst)
    --print(inst, "ShouldSleep", DefaultSleepTest(inst), not inst.sg:HasStateTag("open"), inst.components.follower:IsNearLeader(SLEEP_NEAR_LEADER_DISTANCE))
    return DefaultSleepTest(inst)  and ( inst.components.follower:IsNearLeader(SLEEP_NEAR_LEADER_DISTANCE) or inst.components.follower.leader == nil )

end

local function ShouldKeepTarget()
    return false -- chester can't attack, and won't sleep if he has a target
end





-- eye bone was killed/destroyed
local function OnStopFollowing(inst)
    --print("chester - OnStopFollowing")
    inst:RemoveTag("companion")
end

local function OnStartFollowing(inst)
    --print("chester - OnStartFollowing")
    inst:AddTag("companion")
end



local function OnSave(inst, data)
 
end

local function OnPreLoad(inst, data)
    if data == nil then
        return

    end
end



local function OnHaunt(inst)
    if math.random() <= TUNING.HAUNT_CHANCE_ALWAYS then
        inst.components.hauntable.panic = true
        inst.components.hauntable.panictimer = TUNING.HAUNT_PANIC_TIME_SMALL
        inst.components.hauntable.hauntvalue = TUNING.HAUNT_SMALL
        return true
    end
    return false
end

local function OnAttacked (inst,data)
		local attacker = data.attacker
		if not attacker:HasTag("nuko_killer") then attacker:AddTag("nuko_killer") print("凶手是你！") end 
		inst:DoTaskInTime(20,function() if attacker:HasTag("nuko_killer") then attacker:RemoveTag("nuko_killer") print("警报解除")end end  )
	
end 

local function OnEat(inst, food)
    if food.components.edible ~= nil and math.random(100) > 25  then
            SpawnPrefab("guano").Transform:SetPosition(inst.Transform:GetWorldPosition())
    end
end

local function create_nuko()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()
    inst.entity:AddLightWatcher()

    MakeCharacterPhysics(inst, 75, .5)
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)

    inst:AddTag("companion")
    inst:AddTag("character")
    inst:AddTag("scarytoprey")
    inst:AddTag("nuko")
    inst:AddTag("notraptrigger")
    inst:AddTag("noauradamage")

   --[[ inst.MiniMapEntity:SetIcon("chester.png")
    inst.MiniMapEntity:SetCanUseCache(false)--]]

    inst.AnimState:SetBank("nuko")
    inst.AnimState:SetBuild("nuko")

    inst.DynamicShadow:SetSize(2, 1.5)

    inst.Transform:SetFourFaced()


    inst.entity:SetPristine()

    if not TheWorld.ismastersim then

        return inst
    end

    ------------------------------------------
    inst:AddComponent("maprevealable")
    inst.components.maprevealable:SetIconPrefab("globalmapiconunderfog")

    inst:AddComponent("combat")
    --inst.components.combat.hiteffectsymbol = "body"
    inst.components.combat:SetKeepTargetFunction(ShouldKeepTarget)

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.CHESTER_HEALTH)
    inst.components.health:StartRegen(TUNING.CHESTER_HEALTH_REGEN_AMOUNT, TUNING.CHESTER_HEALTH_REGEN_PERIOD)

    inst:AddComponent("inspectable")
	inst.components.inspectable:SetDescription("他是尤莉的好伙伴~")

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 0.5
    inst.components.locomotor.runspeed = 1

    inst:AddComponent("follower")
    inst:ListenForEvent("stopfollowing", OnStopFollowing)
    inst:ListenForEvent("startfollowing", OnStartFollowing)

    inst:AddComponent("knownlocations")

    --MakeSmallBurnableCharacter(inst, "chester_body")
    inst:AddComponent("eater")
    inst.components.eater:SetDiet({ FOODGROUP.OMNI }, { FOODGROUP.OMNI })
    inst.components.eater:SetCanEatHorrible()
    inst.components.eater:SetCanEatRaw()
	inst.components.eater:SetCanEatGears()
    inst.components.eater.strongstomach = true -- can eat monster meat!
	inst.components.eater:SetOnEatFn(OnEat)


    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(3)
    inst.components.sleeper.testperiod = GetRandomWithVariance(6, 2)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWakeUp)
	inst:ListenForEvent("attacked", OnAttacked)
    MakeHauntableDropFirstItem(inst)
    AddHauntableCustomReaction(inst, OnHaunt, false, false, true)
	
	inst:AddComponent("periodicspawner")
    inst.components.periodicspawner:SetPrefab("guano")
    inst.components.periodicspawner:SetRandomTimes(40, 60)
    inst.components.periodicspawner:SetDensityInRange(20, 2)
    inst.components.periodicspawner:SetMinimumSpacing(8)
    inst.components.periodicspawner:Start()

    inst.sounds = sounds

    inst:SetStateGraph("SGnuko")
    inst.sg:GoToState("idle")

    inst:SetBrain(brain)
	



    inst.OnSave = OnSave
    inst.OnPreLoad = OnPreLoad

    return inst
end

return Prefab("nuko", create_nuko, assets, prefabs)
